/*
 * autotimer.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: autotimer.c 1.0 2004/04/05 13:36:00 hflor Exp $
 */

#include "autotimer.h"
#include "autotimers.h"
#include "menusetup.h"
#include "menuautotimer.h"

static const char *VERSION           = "0.1.0";
static const char *DESCRIPTION       = "AutoTimer-Edit-Interface";

// Global variables that control the overall behaviour:

char    plugin_name[MaxFileName]     = "AutoTimer";

// default parameter
char    d_autotimerfile[MaxFileName] = "/etc/vdradmin/vdradmind.at";
int     d_commandline_preference     = true;
char    d_mainmenu_name[MAXOSDWIDTH] = "Autotimer";
int     d_mainmenu_visible           = true;
int     d_show_channel               = true;
int     d_show_channelname           = true;
int     d_show_flags                 = true;
int     d_show_startstop             = 0x0;
int     d_use_done                   = false;
int     d_verbose                    = false;

// read parameter from commandline
int     c_autotimerfile              = false;
int     c_mainmenu_name              = false;
int     c_mainmenu_visible           = false;
int     c_show_channel               = false;
int     c_show_channelname           = false;
int     c_show_flags                 = false;
int     c_show_startstop             = false;
int     c_use_done                   = false;
int     c_verbose                    = false;

// value parameter from config-file
char    s_autotimerfile[MaxFileName];
int     s_commandline_preference;
char    s_mainmenu_name[MAXOSDWIDTH];
int     s_mainmenu_visible;
int     s_show_channel;
int     s_show_channelname;
int     s_show_flags;
int     s_show_startstop;
int     s_use_done;
int     s_verbose;

// read parameter from config-file
int     r_autotimerfile              = false;
int     r_commandline_preference     = false;
int     r_mainmenu_name              = false;
int     r_mainmenu_visible           = false;
int     r_show_channel               = false;
int     r_show_channelname           = false;
int     r_show_flags                 = false;
int     r_show_startstop             = false;
int     r_use_done                   = false;
int     r_verbose                    = false;

// paramater used
char    u_autotimerfile[MaxFileName];
int     u_commandline_preference;
char    u_mainmenu_name[MAXOSDWIDTH];
int     u_mainmenu_visible;
int     u_show_channel;
int     u_show_channelname;
int     u_show_flags;
int     u_show_startstop             = 0;
int     u_use_done;
int     u_verbose;

const char * StartStopTime[4];

int GetColumn(int selpos)
{
  int column[5] = { 2, 0, 0, 0, 0};
  int pos = 1;
  if (u_show_flags)
    column[pos++] = u_use_done ? 7 : 5;
  if (u_show_channel)
    column[pos++] = u_show_channelname ? 8 : numdigits(Channels.MaxNumber()) + 1;
  if (u_show_startstop & 0x1)
    column[pos++] = 6;
  if (u_show_startstop & 0x2)
    column[pos++] = 6;
  return column[selpos];
}

void DisplaySetings(void)
{
  if (u_verbose) {
    isyslog("%s: autotimerfile          = %c%s", plugin_name, strcmp(u_autotimerfile, d_autotimerfile) ? ' ' : '*', u_autotimerfile);
    isyslog("%s: commandline_preference = %c%s", plugin_name, u_commandline_preference == d_commandline_preference ? '*' : ' ', u_commandline_preference ? "yes" : "no");
    isyslog("%s: mainmenu_visible       = %c%s", plugin_name, u_mainmenu_visible == d_mainmenu_visible ? '*' : ' ', u_mainmenu_visible ? "yes" : "no");
    if (u_mainmenu_visible)
      isyslog("%s: mainmenu_name          = %c%s", plugin_name, strcmp(u_mainmenu_name, d_mainmenu_name) ? ' ' : '*', u_mainmenu_name);
    isyslog("%s: show_channel           = %c%s", plugin_name, u_show_channel == d_show_channel ? '*' : ' ', u_show_channel ? "yes" : "no");
    if (u_show_channel)
      isyslog("%s: show_channelname       = %c%s", plugin_name, u_show_channelname == d_show_channelname ? '*' : ' ', u_show_channelname ? "yes" : "no");
    isyslog("%s: show_flags             = %c%s", plugin_name, u_show_flags == d_show_flags ? '*' : ' ', u_show_flags ? "yes" : "no");
    isyslog("%s: show_startstop         = %c%s", plugin_name, u_show_startstop == d_show_startstop ? '*' : ' ', StartStopTime[u_show_startstop]);
    isyslog("%s: use_done               = %c%s", plugin_name, u_use_done == d_use_done ? '*' : ' ', u_use_done ? "yes" : "no");
  }
}

class cPluginAutoTimer : public cPlugin {
private:
  // Add any member variables or functions you may need here.
public:
  cPluginAutoTimer(void);
  virtual ~cPluginAutoTimer();
  virtual const char *Version(void)                             { return VERSION; }
  virtual const char *Description(void)                         { return tr(DESCRIPTION); }
  virtual const char *CommandLineHelp(void);
  virtual bool ProcessArgs(int argc, char *argv[]);
  virtual bool Start(void);
  virtual const char *MainMenuEntry(void)                       { return (u_mainmenu_visible) ? u_mainmenu_name : NULL; }
  virtual cOsdObject *MainMenuAction(void)                      { return new cMenuAutoTimers; }
  virtual cMenuSetupPage *SetupMenu(void)                       { return new cMenuSetupAutoTimer; }
  virtual bool SetupParse(const char *Name, const char *Value);
  };

// --- cPluginAutoTimer ----------------------------------------------------------

cPluginAutoTimer::cPluginAutoTimer(void)
{
  // Initialize any member variables here.
  // DON'T DO ANYTHING ELSE THAT MAY HAVE SIDE EFFECTS, REQUIRE GLOBAL
  // VDR OBJECTS TO EXIST OR PRODUCE ANY OUTPUT!
}

cPluginAutoTimer::~cPluginAutoTimer()
{
  // Clean up after yourself!
}

const char *cPluginAutoTimer::CommandLineHelp(void)
{
  // Return a string that describes all known command line options.
  return "  -f path  --autotimerfile=path     Select Autotimer-file (same for vdradmind!)\n"
         "  -m       --visible_in_mainmenu    Show the plugin in the mainmenu\n"
         "  -M       --hide_in_mainmenu       Hide the plugin in the mainmenu\n"
         "  -n Name  --mainmenu_name=Name     Select Name for entry in the mainmenu (set also -m)\n"
         "  -v       --verbose                Enable more logging\n"
         "  -V       --noverbose              Disable more loggig\n"
         "  -r       --use_repeat_feature     use the repeat feature from vdradmin bigpatch 0.9\n"
         "  -R       --not_use_repeat_feature not use, it set automatic to use, if the autotimerfile with 11 values\n"
         "  -l       --show_flags             For the Autotimerlist show search in title, subtitle and description as a single char\n"
         "  -L       --hide_flags             column is hide\n"
         "  -a       --show_start             For the Autotimerlist show the startsearchtime\n"
         "  -A       --hide_start             column is hide\n"
         "  -t       --show_stop              For the Autotimerlist show the stopsearchtime\n"
         "  -T       --hide_stop              column is hide\n"
         "  -c       --show_channel           For the Autotimerlist show the searchchannel\n"
         "  -C       --hide_channel           column is hide\n"
         "  -s       --show_channel_name      if show_channel the name of channel is use\n"
         "  -S       --show_channel_number    if show_channel the number of channel is use\n";
}

bool cPluginAutoTimer::ProcessArgs(int argc, char *argv[])
{
  int c;
  // Implement command line argument processing here if applicable.
  static struct option long_options[] = {
    { "autotimerfile",          required_argument, NULL, 'f' },
    { "visible_in_mainmenu",    no_argument,       NULL, 'm' },
    { "hide_in_mainmenu",       no_argument,       NULL, 'M' },
    { "mainmenu_name",          required_argument, NULL, 'n' },
    { "verbose",                no_argument,       NULL, 'v' },
    { "noverbose",              no_argument,       NULL, 'V' },
    { "use_repeat_feature",     no_argument,       NULL, 'r' },
    { "not_use_repeat_feature", no_argument,       NULL, 'R' },
    { "show_flags",             no_argument,       NULL, 'l' },
    { "hide_flags",             no_argument,       NULL, 'L' },
    { "show_start",             no_argument,       NULL, 'a' },
    { "hide_start",             no_argument,       NULL, 'A' },
    { "show_stop",              no_argument,       NULL, 't' },
    { "hide_stop",              no_argument,       NULL, 'T' },
    { "show_channel",           no_argument,       NULL, 'c' },
    { "hide_channel",           no_argument,       NULL, 'C' },
    { "show_channel_name",      no_argument,       NULL, 's' },
    { "show_channel_number",    no_argument,       NULL, 'S' },
    { NULL }
  };

  if (argc >= 1)
    strn0cpy(plugin_name, argv[0], sizeof(plugin_name));

  for (c = 1; c < argc; c++) {
    dsyslog("%s: Parameter%d=%s", plugin_name, c + 1, argv[c]);
  }

  while ((c = getopt_long(argc, argv, "n:v:V", long_options, NULL)) != -1) {
    switch (c) {
      case 'a': c_show_startstop = true;
                u_show_startstop |= 0x1;
                break;
      case 'A': c_show_startstop = true;
                u_show_startstop &= 0xfffffffe;
                break;
      case 'c': c_show_channel = true;
                u_show_channel = true;
                break;
      case 'C': c_show_channel = true;
                u_show_channel = false;
                break;
      case 'f': c_autotimerfile = true;
                strn0cpy(u_autotimerfile, optarg, sizeof(u_autotimerfile));
                break;
      case 'l': c_show_flags = true;
                u_show_flags = true;
                break;
      case 'L': c_show_flags = true;
                u_show_flags = false;
                break;
      case 'm': c_mainmenu_visible = true;
                u_mainmenu_visible = true;
                break;
      case 'M': c_mainmenu_visible = true;
                u_mainmenu_visible = false;
                break;
      case 'n': c_mainmenu_name = true;
                strn0cpy(u_mainmenu_name, optarg, sizeof(u_mainmenu_name));
                c_mainmenu_visible = true;
                u_mainmenu_visible = true;
                break;
      case 'r': c_use_done = true;
                u_use_done = true;
                break;
      case 'R': c_use_done = true;
                u_use_done = false;
                break;
      case 's': c_show_channelname = true;
                u_show_channelname = true;
                break;
      case 'S': c_show_channelname = true;
                u_show_channelname = false;
                break;
      case 't': c_show_startstop = true;
                u_show_startstop |= 0x2;
                break;
      case 'T': c_show_startstop = true;
                u_show_startstop &= 0xfffffffd;
                break;
      case 'v': c_verbose = true;
                u_verbose = true;
                break;
      case 'V': c_verbose = true;
                u_verbose = false;
                break;
      default:  return false;
    }
  }
  return true;
}

bool cPluginAutoTimer::Start(void)
{
  // Start any background activities the plugin shall perform.
  RegisterI18n(Phrases);
  StartStopTime[0] = tr("AutoTimer.Time$none");
  StartStopTime[1] = tr("AutoTimer.Time$start Time");
  StartStopTime[2] = tr("AutoTimer.Time$stop Time");
  StartStopTime[3] = tr("AutoTimer.Time$both Times");

  // default parameter                --> d_
  // read parameter from commandline  --> c_  (data in u_)
  // value parameter from config-file --> s_
  // read parameter from config-file  --> r_
  // paramater used                   --> u_
  // if (!c || (!u_commandline_preference && r)) u = (!c && !r) ? d : s;
  u_commandline_preference = r_commandline_preference ? s_commandline_preference : d_commandline_preference;
  if (!c_autotimerfile    || (!u_commandline_preference && r_autotimerfile))    strcpy(u_autotimerfile, (!c_autotimerfile && !r_autotimerfile)    ? d_autotimerfile    : s_autotimerfile);
  if (!c_mainmenu_name    || (!u_commandline_preference && r_mainmenu_name))    strcpy(u_mainmenu_name, (!c_mainmenu_name && !r_mainmenu_name)    ? d_mainmenu_name    : s_mainmenu_name);
  if (!c_mainmenu_visible || (!u_commandline_preference && r_mainmenu_visible)) u_mainmenu_visible = (!c_mainmenu_visible && !r_mainmenu_visible) ? d_mainmenu_visible : s_mainmenu_visible;
  if (!c_show_channel     || (!u_commandline_preference && r_show_channel))     u_show_channel     = (!c_show_channel     && !r_show_channel)     ? d_show_channel     : s_show_channel;
  if (!c_show_channelname || (!u_commandline_preference && r_show_channelname)) u_show_channelname = (!c_show_channelname && !r_show_channelname) ? d_show_channelname : s_show_channelname;
  if (!c_show_flags       || (!u_commandline_preference && r_show_flags))       u_show_flags       = (!c_show_flags       && !r_show_flags)       ? d_show_flags       : s_show_flags;
  if (!c_show_startstop   || (!u_commandline_preference && r_show_startstop))   u_show_startstop   = (!c_show_startstop   && !r_show_startstop)   ? d_show_startstop   : s_show_startstop;
  if (!c_use_done         || (!u_commandline_preference && r_use_done))         u_use_done         = (!c_use_done         && !r_use_done)         ? d_use_done         : s_use_done;
  if (!c_verbose          || (!u_commandline_preference && r_verbose))          u_verbose          = (!c_verbose          && !r_verbose)          ? d_verbose          : s_verbose;
  if (u_verbose)
    isyslog("%s: Start", plugin_name);
  dsyslog("%s: Start", plugin_name);
  DisplaySetings();
  return true;
}

bool cPluginAutoTimer::SetupParse(const char *Name, const char *Value)
{
  // Parse your own setup parameters and store their values.
  dsyslog("%s: Setupparameter %s=%s", plugin_name, Name, Value);
  if      (!strcasecmp(Name, "autotimerfile"))    { r_autotimerfile = true;          strn0cpy(s_autotimerfile, Value, sizeof(s_autotimerfile)); }
  else if (!strcasecmp(Name, "commandline"))      { r_commandline_preference = true; s_commandline_preference = atoi(Value); }
  else if (!strcasecmp(Name, "name"))             { r_mainmenu_name = true;          strn0cpy(s_mainmenu_name, Value, sizeof(s_mainmenu_name)); }
  else if (!strcasecmp(Name, "show_channel"))     { r_show_channel = true;           s_show_channel = atoi(Value); }
  else if (!strcasecmp(Name, "show_channelname")) { r_show_channelname = true;       s_show_channelname = atoi(Value); }
  else if (!strcasecmp(Name, "show_flags"))       { r_show_flags = true;             s_show_flags = atoi(Value); }
  else if (!strcasecmp(Name, "show_startstop"))   { r_show_startstop = true;         s_show_startstop = atoi(Value); }
  else if (!strcasecmp(Name, "use_done"))         { r_use_done = true;               s_use_done = atoi(Value); }
  else if (!strcasecmp(Name, "verbose"))          { r_verbose = true;                s_verbose = atoi(Value); }
  else if (!strcasecmp(Name, "visible"))          { r_mainmenu_visible = true;       s_mainmenu_visible = atoi(Value); }
  else
     return false;
  return true;
}

VDRPLUGINCREATOR(cPluginAutoTimer); // Don't touch this!
