/*
 * autotimer.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: autotimers.c 1.0 2004/04/05 13:36:00 hflor Exp $
 */

#include "autotimer.h"
#include "autotimers.h"

// -- cAutoTimer -----------------------------------------------------------------

char *cAutoTimer::buffer = NULL;

cAutoTimer::cAutoTimer(bool Instant, bool Pause)
{
  active = true;
  *search = 0;
  options = 0;
  title = true;
  subtitle = false;
  description = false;
  useStart = false;
  startTime = 0000;
  useStop = false;
  stopTime = 2359;
  serie = false;
  priority = Setup.DefaultPriority;
  lifetime = Setup.DefaultLifetime;
  useChannel = false;
  channel = Channels.GetByNumber(cDevice::CurrentChannel());
  *folder = 0;
  done = false;
}

cAutoTimer& cAutoTimer::operator= (const cAutoTimer &AutoTimer)
{
  memcpy(this, &AutoTimer, sizeof(*this));
  return *this;
}

bool cAutoTimer::operator< (const cListObject &ListObject)
{
  cAutoTimer *AT = (cAutoTimer *)&ListObject;
  return strcasecmp(search, AT->search) < 0;
}

const char *cAutoTimer::ToText(void)
{
  char *tmp_Start = "";
  char *tmp_Stop = "";
  char *tmp_Done = "";

  free(buffer);
  strreplace(search, ':', '|');
  strreplace(folder, ':', '|');
  
  if (useStart)
    asprintf(&tmp_Start, "%04d", startTime);
  if (useStop)
    asprintf(&tmp_Stop, "%04d", stopTime);
  if (u_use_done)
    asprintf(&tmp_Done, ":%d", done);
  
  asprintf(&buffer, "%d:%s:%d:%s:%s:%d:%d:%d:%d:%s%s\n",
                    active,
                    search,
                    (options & 0xfffffff8) | (title ? 1 : 0) | (subtitle ? 2 : 0) | (description ? 4 : 0),
                    tmp_Start,
                    tmp_Stop,
                    serie,
                    priority,
                    lifetime,
                    useChannel ? channel->Number() : 0,
                    folder,
                    tmp_Done);

  strreplace(search, '|', ':');
  strreplace(folder, '|', ':');

  if (u_verbose)
    isyslog("%s: write line [%.*s]", plugin_name, (int)strlen(buffer) - 1, buffer);
  return buffer;
}

bool cAutoTimer::Parse(const char *s)
{
  char *line;
  char *pos;
  char *pos_next;
  int parameter = 1;
  int valuelen;
  char value[MaxFileName];
  tChannelID cid;
  
  if (u_verbose)
    isyslog("%s: readline [%s]", plugin_name, s);

  pos = line = strdup(s);
  pos_next = pos + strlen(pos);
  if (*pos_next == '\n') *pos_next = 0;
  while (*pos) {
    while (*pos == ' ') pos++;
    if (*pos) {
      if (*pos != ':') {
        pos_next = strchr(pos, ':');
        if (!pos_next)
          pos_next = pos + strlen(pos);
        valuelen = pos_next - pos + 1;
        if (valuelen > MaxFileName) valuelen = MaxFileName;
        strn0cpy(value, pos, valuelen);
        pos = pos_next;
        switch (parameter) {
          case 1:  active = atoi(value);
                   break;
          case 2:  strcpy(search, value);
                   break;
          case 3:  options = atoi(value);
                   title = options & 0x1 ? true : false;
                   subtitle = options & 0x2 ? true : false;
                   description = options & 0x4 ? true : false;
                   break;
          case 4:  useStart = true;
                   startTime = atoi(value);
                   break;
          case 5:  useStop = true;
                   stopTime = atoi(value);
                   break;
          case 6:  serie = atoi(value) != 0 ? true : false;
                   break;
          case 7:  priority = atoi(value);
                   if (priority < 0) priority = 0;
                   if (priority > MAXPRIORITY) priority = MAXPRIORITY;
                   break;
          case 8:  lifetime = atoi(value);
                   if (lifetime < 0) lifetime = 0;
                   if (lifetime > MAXLIFETIME) lifetime = MAXLIFETIME;
                   break;
          case 9:  if (atoi(value)) {
                     useChannel = true;
                     cid = tChannelID::FromString(value);
                     channel = cid.Valid() ? Channels.GetByChannelID(cid, true) : Channels.GetByNumber(atoi(value));
                     if (!channel) {
                       esyslog("ERROR: channel %s not defined", value);
                       free(line);
                       return false;
                     }
                   }
                   break;
          case 10: strcpy(folder, value);
                   break;
          case 11: done = atoi(value) != 0 ? true : false;
                   if (u_verbose && !u_use_done)
                     isyslog("%s: auto enable except repeat feature", plugin_name);
                   u_use_done = true;
                   break;
        } //switch
      }
      parameter++;
    }
    if (*pos) pos++;
  } //while

  strreplace(search, '|', ':');
  strreplace(folder, '|', ':');

  free(line);
  return (parameter >= 10) ? true : false;
}

bool cAutoTimer::Save(FILE *f)
{
  return fprintf(f, ToText()) > 0;
}

void cAutoTimer::OnOff(void)
{
  active = !active;
}

// -- cAutoTimers ----------------------------------------------------------------

cAutoTimers AutoTimers;
