/*
 * autotimer.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: menuautotimer.c 1.0 2004/04/05 13:36:00 hflor Exp $
 */

#include "autotimer.h"
#include "menuautotimer.h"

// --- cMenuEditAutoTimer --------------------------------------------------------

cMenuEditAutoTimer::cMenuEditAutoTimer(cAutoTimer *AutoTimer, bool New)
:cOsdMenu(tr("Autotimer.Edit$Edit Autotimer"), 25)
{
  autotimer = AutoTimer;
  addIfConfirmed = New;
  if (autotimer) {
     data = *autotimer;
     selStartStopTime = (data.useStart ? 0x1 : 0) | (data.useStop ? 0x2 : 0);
     channel = data.channel->Number();
     Set();
  }
}

cMenuEditAutoTimer::~cMenuEditAutoTimer()
{
  if (autotimer && addIfConfirmed)
    delete autotimer; // apparently it wasn't confirmed
}

void cMenuEditAutoTimer::Set(void)
{
  int current = Current();
  Clear();
  char trans_channel[MAXOSDWIDTH + 1] = "  ";
  strn0cpy(&trans_channel[2], tr("Channel"), MAXOSDWIDTH-1);

  Add(new cMenuEditBoolItem(   tr("Active"),                               &data.active));
  Add(new cMenuEditStrItem(    tr("AutoTimer.Edit$Search Text"),           data.search, sizeof(data.search), tr(FileNameChars)));
  Add(new cMenuEditBoolItem(   tr("AutoTimer.Edit$Search in Title"),       &data.title));
  Add(new cMenuEditBoolItem(   tr("AutoTimer.Edit$Search in Subtitle"),    &data.subtitle));
  Add(new cMenuEditBoolItem(   tr("AutoTimer.Edit$Search in Description"), &data.description));
  Add(new cMenuEditStraItem(   tr("AutoTimer.Edit$Use start/stop Time"),   &selStartStopTime, 4, StartStopTime));
  if (selStartStopTime & 0x1)
    Add(new cMenuEditTimeItem( tr("AutoTimer.Edit$  Search start Time"),   &data.startTime));
  if (selStartStopTime & 0x2)
    Add(new cMenuEditTimeItem( tr("AutoTimer.Edit$  Search stop Time"),    &data.stopTime));
  Add(new cMenuEditBoolItem(   tr("AutoTimer.Edit$Use Channel select"),    &data.useChannel));
  if (data.useChannel)
    Add(new cMenuEditChanItem( trans_channel,                              &channel));
  Add(new cMenuEditStrItem(    tr("AutoTimer.Edit$Folder for Record"),     data.folder, sizeof(data.folder), tr(FileNameChars)));
  Add(new cMenuEditBoolItem(   tr("AutoTimer.Edit$Series select"),         &data.serie));
  if (u_use_done)
    Add(new cMenuEditBoolItem( tr("AutoTimer.Edit$Except repeat"),         &data.done));
  Add(new cMenuEditIntItem(    tr("Priority"),                             &data.priority, 0, MAXPRIORITY));
  Add(new cMenuEditIntItem(    tr("Lifetime"),                             &data.lifetime, 0, MAXLIFETIME));

  SetCurrent(Get(current));
}

eOSState cMenuEditAutoTimer::ProcessKey(eKeys Key)
{
  int m_selStartStopTime = selStartStopTime;
  int m_useChannel = data.useChannel;

  eOSState state = cOsdMenu::ProcessKey(Key);

  if (m_selStartStopTime != selStartStopTime || m_useChannel != data.useChannel) {
    Set();
    Display();
  }

  if (state == osUnknown) {
    switch (Key) {
      case kOk:     {
                      if (!strlen(data.search)) {
                        Interface->Error(tr("AutoTimer.Error$Missing search text!"));
                        break;
                      }
                      if (data.useChannel) {
                        cChannel *ch = Channels.GetByNumber(channel);
                        if (ch)
                          data.channel = ch;
                        else {
                          Interface->Error(tr("*** Invalid Channel ***"));
                          break;
                        }
                      }
                      data.useStart = (selStartStopTime & 0x1) ? true : false;
                      data.useStop = (selStartStopTime & 0x2) ? true : false;
                      if (autotimer) {
                        *autotimer = data;
                        if (addIfConfirmed)
                          AutoTimers.Add(autotimer);
                        AutoTimers.Save();
                        isyslog("autotimer %d %s (%s)", autotimer->Index() + 1, addIfConfirmed ? "added" : "modified", autotimer->active ? "active" : "inactive");
                        addIfConfirmed = false;
                      }
                      return osBack;
                    }
      case kRed:
      case kGreen:
      case kYellow:
      case kBlue:   return osContinue;
      default: break;
    }
  }
  return state;
}

// --- cMenuAutoTimerItem --------------------------------------------------------

cMenuAutoTimerItem::cMenuAutoTimerItem(cAutoTimer *AutoTimer)
{
  autotimer = AutoTimer;
  Set();
}

bool cMenuAutoTimerItem::operator< (const cListObject &ListObject)
{
  return *autotimer < *((cMenuAutoTimerItem *)&ListObject)->autotimer;
}

void cMenuAutoTimerItem::Set(void)
{
  char *buffer = NULL;
  char *tmp[]  = { "", "", "", "" };
  int pos = 0;
  if (u_show_flags)
    asprintf(&tmp[pos++], "\t%s%s%s%s",
                          autotimer->Title() ? tr("AutoTimer.Display1$T") : "-",
                          autotimer->Subtitle() ? tr("AutoTimer.Display2$S") : "-",
                          autotimer->Description() ? tr("AutoTimer.Display3$D") : "-",
                          u_use_done ? autotimer->Done() ? tr("AutoTimer.Display4$R") : "-" : "");
  if (u_show_channel)
    if (autotimer->UseChannel()) {
      if (u_show_channelname)
        asprintf(&tmp[pos++], "\t%.7s", autotimer->Channel()->Name());
      else
        asprintf(&tmp[pos++], "\t%0*d", numdigits(Channels.MaxNumber()), autotimer->Channel()->Number());
    } else
      asprintf(&tmp[pos++], "\t--");
  if (u_show_startstop & 0x1) {
    if (autotimer->UseStart())
      asprintf(&tmp[pos++], "\t%02d:%02d", autotimer->StartTime() / 100, autotimer->StartTime() % 100);
    else
      asprintf(&tmp[pos++], "\t--:--");
  }
  if (u_show_startstop & 0x2) {
    if (autotimer->UseStop())
      asprintf(&tmp[pos++], "\t%02d:%02d", autotimer->StopTime() / 100, autotimer->StopTime() % 100);
    else
      asprintf(&tmp[pos++], "\t--:--");
  }

  asprintf(&buffer, "%c%s%s%s%s\t%s", autotimer->Active() ? ICON_UHR : ' ', tmp[0], tmp[1], tmp[2], tmp[3], autotimer->Search());
  SetText(buffer, false);
}

// --- cMenuAutoTimers -----------------------------------------------------------

cMenuAutoTimers::cMenuAutoTimers(void)
:cOsdMenu(tr("AutoTimer.Display$AutoTimers"), GetColumn(0), GetColumn(1), GetColumn(2), GetColumn(3), GetColumn(4))
{
  int i = 0;
  cAutoTimer *autotimer;

  AutoTimers.Load(u_autotimerfile);
  while ((autotimer = AutoTimers.Get(i++)) != NULL)
    Add(new cMenuAutoTimerItem(autotimer));
  Sort();
  SetHelpKeys();
}

void cMenuAutoTimers::SetHelpKeys(void)
{
  if (Count())
    SetHelp(tr("Edit"), tr("New"), tr("Delete"), tr("On/Off"));
  else
    SetHelp(NULL, tr("New"), NULL, NULL);
}

cAutoTimer *cMenuAutoTimers::CurrentAutoTimer(void)
{
  cMenuAutoTimerItem *item = (cMenuAutoTimerItem *)Get(Current());
  return item ? item->AutoTimer() : NULL;
}

eOSState cMenuAutoTimers::OnOff(void)
{
  cAutoTimer *autotimer = CurrentAutoTimer();
  if (autotimer) {
     autotimer->OnOff();
     RefreshCurrent();
     DisplayCurrent(true);
     isyslog("autotimer %d %sactivated", autotimer->Index() + 1, autotimer->Active() ? "" : "de");
     AutoTimers.Save();
     }
  return osContinue;
}

eOSState cMenuAutoTimers::Edit(void)
{
  if (HasSubMenu() || Count() == 0)
    return osContinue;
  isyslog("editing autotimer %d", CurrentAutoTimer()->Index() + 1);
  return AddSubMenu(new cMenuEditAutoTimer(CurrentAutoTimer()));
}

eOSState cMenuAutoTimers::New(void)
{
  if (HasSubMenu())
    return osContinue;
  eOSState state = AddSubMenu(new cMenuEditAutoTimer(new cAutoTimer, true));
  Sort();
  return state;
}

eOSState cMenuAutoTimers::Delete(void)
{
  // Check if this autotimer is active:
  cAutoTimer *ti = CurrentAutoTimer();
  if (ti) {
    if (Interface->Confirm(tr("AutoTimer.Display$Delete Autotimer?"))) {
      int Index = ti->Index();
      AutoTimers.Del(ti);
      cOsdMenu::Del(Current());
      AutoTimers.Save();
      Display();
      isyslog("autotimer %d deleted", Index + 1);
    }
  }
  return osContinue;
}

eOSState cMenuAutoTimers::ProcessKey(eKeys Key)
{
  cOsdItem *Item;
  int n;
  int AutoTimerNumber = HasSubMenu() ? Count() : -1;
  
  eOSState state = cOsdMenu::ProcessKey(Key);

  if (state == osUnknown) {
    switch (Key) {
      case k0...k9: n = (Key == k0) ? 10 : (Key - k0);
                    if (n > Count()) break;
                    SetCurrent(Get(--n));
                    // no break!
      case kOk:
      case kRed:    state = Edit(); break;
      case kGreen:  state = New(); break;
      case kYellow: state = Delete(); break;
      case kBlue:   state = OnOff(); break;
      default: break;
    }
  }
  if (AutoTimerNumber >= 0 && !HasSubMenu() && AutoTimers.Get(AutoTimerNumber)) {
    // a newly created autotimer was confirmed with Ok
    Add((Item = new cMenuAutoTimerItem(AutoTimers.Get(AutoTimerNumber))), true);
    Sort();
    SetCurrent(Item);
    Display();
  }

  if (!HasSubMenu() && Key != kNone)
    SetHelpKeys();

  return state;
}
