/*
 * autotimer.c: A plugin for the Video Disk Recorder
 *
 * See the README file for copyright information and how to reach the author.
 *
 * $Id: menusetup.c 1.0 2004/04/05 13:36:00 hflor Exp $
 */

#include "menusetup.h"
#include "menuautotimer.h"

// --- cMenuSetupAutoTimer -------------------------------------------------------

cMenuSetupAutoTimer::cMenuSetupAutoTimer(void)
{
  show_setup = false;
  store = false;
  strcpy(old_autotimerfile, u_autotimerfile);
  old_commandline_preference = u_commandline_preference;
  strcpy(old_mainmenu_name, u_mainmenu_name);
  old_mainmenu_visible = u_mainmenu_visible;
  old_show_channel = u_show_channel;
  old_show_channelname = u_show_channelname;
  old_show_flags = u_show_flags;
  old_show_startstop = u_show_startstop;
  old_use_done = u_use_done;
  old_verbose = u_verbose;
  Set();
}

cMenuSetupAutoTimer::~cMenuSetupAutoTimer(void)
{
  if (!store) {
    strcpy(u_autotimerfile, old_autotimerfile);
    u_commandline_preference = old_commandline_preference;
    strcpy(u_mainmenu_name, old_mainmenu_name);
    u_mainmenu_visible = old_mainmenu_visible;
    u_show_channel = old_show_channel;
    u_show_channelname = old_show_channelname;
    u_show_flags = old_show_flags;
    u_show_startstop = old_show_startstop;
    u_use_done = old_use_done;
    u_verbose = old_verbose;
  }
}

void cMenuSetupAutoTimer::Store(void)
{
  if (strcmp(u_autotimerfile, old_autotimerfile))
    SetupStore("autotimerfile", u_autotimerfile);
  if (u_commandline_preference != old_commandline_preference)
    SetupStore("commandline", u_commandline_preference);
  if (strcmp(u_mainmenu_name, old_mainmenu_name))
    SetupStore("name", u_mainmenu_name);
  if (u_show_channel != old_show_channel)
    SetupStore("show_channel", u_show_channel);
  if (u_show_channelname != old_show_channelname)
    SetupStore("show_channelname", u_show_channelname);
  if (u_show_flags != old_show_flags)
    SetupStore("show_flags", u_show_flags);
  if (u_show_startstop != old_show_startstop)
    SetupStore("show_startstop", u_show_startstop);
  if (u_use_done != old_use_done)
    SetupStore("use_done", u_use_done);
  if (u_verbose != old_verbose)
    SetupStore("verbose", u_verbose);
  if (u_mainmenu_visible != old_mainmenu_visible)
    SetupStore("visible", u_mainmenu_visible);
  store = true;
}

void cMenuSetupAutoTimer::Set(void)
{
  int current = Current();
  Clear();

  Add(new cMenuEditBoolItem(tr("AutoTimer.Setup$Preferr Command Line Parameter"),       &u_commandline_preference));
  Add(new cMenuEditStrItem (tr("AutoTimer.Setup$Path for vdradmind.at"),                u_autotimerfile, sizeof(u_autotimerfile), tr(FileNameChars)));
  Add(new cMenuEditBoolItem(tr("AutoTimer.Setup$Except repeat Feature (bigpatch 0.9)"), &u_use_done));
  Add(new cMenuEditBoolItem(tr("AutoTimer.Setup$Visible in Mainmenu"),                  &u_mainmenu_visible));
  if (u_mainmenu_visible)
    Add(new cMenuEditStrItem (tr("AutoTimer.Setup$  Name for Mainmenu"),                u_mainmenu_name, sizeof(u_mainmenu_name), tr(FileNameChars)));
  Add(new cMenuEditBoolItem(tr("AutoTimer.Setup$Verbose Log Mode"),                     &u_verbose));
  Add(new cMenuEditBoolItem(tr("AutoTimer.Setup$Setup List Format"),                    &show_setup));
  if (show_setup) {
    Add(new cMenuEditBoolItem(tr("AutoTimer.Setup$  Show Flags"),                       &u_show_flags));
    Add(new cMenuEditBoolItem(tr("AutoTimer.Setup$  Show Channel"),                     &u_show_channel));
    if (u_show_channel)
      Add(new cMenuEditBoolItem(tr("AutoTimer.Setup$    Use Channelname"),              &u_show_channelname));
    Add(new cMenuEditStraItem(   tr("AutoTimer.Setup$  Show start/stop Time"),          &u_show_startstop, 4, StartStopTime));
  }
  Add(new cOsdItem         (tr("AutoTimer.Setup$Edit Autotimer now"),                   os_User));

  Add(new cOsdItem         ("",                                                         osContinue));
  char *comment = strdup(tr("AutoTimer.Setup$For this plugin ist vdradmind requiered!\nIn the config of vdradmind must the autotimer-function be enabled."));
  char *pos = comment;
  char *pos_next;
  char *pos_last;
  char text[MAXOSDWIDTH + 1];
  int textlen;

  while (*pos) {
    pos_next = strchr(pos, '\n');
    if (!pos_next)
      pos_next = pos + strlen(pos);
    textlen = pos_next - pos;
    if (textlen <= Setup.OSDwidth) {
      strn0cpy(text, pos, textlen + 1);
    } else {
      strn0cpy(text, pos, Setup.OSDwidth + 1);
      pos_last = strrchr(text, ' ');
      if (!pos_last)
        pos_last = text + Setup.OSDwidth + 1;
      *pos_last = 0;
      textlen = pos_last - text;
    }
    Add(new cOsdItem(text, osContinue));
    pos += textlen;
    if (*pos) pos++;
  }
  free(comment);
  SetCurrent(Get(current));
}

eOSState cMenuSetupAutoTimer::ProcessKey(eKeys Key)
{
  int m_mainmenu_visible = u_mainmenu_visible;
  int m_show_setup = show_setup;
  int m_show_channel = u_show_channel;
  int pathlen;
  char status[MAXOSDWIDTH];

  eOSState state = cMenuSetupPage::ProcessKey(Key);

  if (m_mainmenu_visible != u_mainmenu_visible || m_show_setup != show_setup || m_show_channel != u_show_channel) {
    Set();
    Display();
  }
  if (Current() == 1) {
    pathlen = strlen(u_autotimerfile);
    if (pathlen > MAXOSDWIDTH - 10) {
      sprintf(status, "...%s", &u_autotimerfile[pathlen - MAXOSDWIDTH + 10]);
    } else
      strcpy(status, u_autotimerfile);
    Interface->Status(status, Setup.Theme == themeVanilla ? clrBlack : (eDvbColor)fginfofont, Setup.Theme == themeVanilla ? clrGreen : clrInfoLine);
  } else
    Interface->Status(NULL);

  if (state == os_User)
    state = AddSubMenu(new cMenuAutoTimers);

  return state;
}
